# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['ip_liberator']

package_data = \
{'': ['*']}

install_requires = \
['boto3==1.9.133']

entry_points = \
{'console_scripts': ['ip-liberator = ip_liberator.__main__:main']}

setup_kwargs = {
    'name': 'ip-liberator',
    'version': '0.2.3',
    'description': 'A command line utility to update AWS Security Groups rules.',
    'long_description': '============\nIP Liberator\n============\n\n\n.. image:: https://img.shields.io/pypi/v/ip-liberator.svg\n        :target: https://pypi.python.org/pypi/ip-liberator\n\n.. image:: https://img.shields.io/travis/wagnerluis1982/ip-liberator.svg\n        :target: https://travis-ci.org/wagnerluis1982/ip-liberator\n\n.. image:: https://readthedocs.org/projects/ip-liberator/badge/?version=latest\n        :target: https://ip-liberator.readthedocs.io/en/latest/?badge=latest\n        :alt: Documentation Status\n\n\nA command line utility to update AWS Security Groups rules.\n\n\n* Free software: GNU General Public License v3\n* Documentation: https://ip-liberator.readthedocs.io.\n\n\nFeatures\n--------\n\n* Update a list of security groups of your AWS account at once.\n* Grant access to informed ports for your current IP address or an informed IP.\n* Read profile files in JSON with all the information needed to contact.\n* Fit for use as script (e.g. to update your dynamic IP regularly).\n\nInstallation\n------------\n\n.. code-block:: console\n\n    $ pip install ip-liberator\n\nQuickstart\n----------\n\nConsider a file ``/path/my-profile.json`` with the following content:\n\n.. code-block:: json\n\n    {\n      "credentials": {\n        "access_key": "<AWS_ACCESS_KEY>",\n        "secret_key": "<AWS_SECRET_KEY>",\n        "region_name": "<AWS REGION>"\n      },\n      "config": {\n        "operator": "John",\n        "services": [\n          {\n            "name": "FTP+SFTP",\n            "port": "21-22"\n          },\n          {\n            "name": "HTTPS",\n            "port": "443"\n          }\n        ],\n        "security_groups": [\n          "sg-<GROUP_ID_1>",\n          "sg-<GROUP_ID_2>"\n        ]\n      }\n    }\n\nUsing the profile defined above will create or update two entries in the informed security groups:\n\n- **John FTP+SFTP** granting access for the current IP the ports 21 and 22.\n- **John HTTPS** granting access for the current IP the port 443.\n\nTo accomplish it, simply run:\n\n.. code-block:: console\n\n    $ ip-liberator --profile /path/my-profile.json\n    Authorizing rules [\'John FTP+SSH\', \'John HTTPS\'] to IP 192.30.253.112/32\n    - sg-<GROUP_ID_1>\n    - sg-<GROUP_ID_2>\n\nCredits\n-------\n\nAuthors\n:::::::\n\n* Wagner Macedo <wagnerluis1982@gmail.com> (maintainer)\n',
    'author': 'Wagner Macedo',
    'author_email': 'wagnerluis1982@gmail.com',
    'url': 'https://github.com/wagnerluis1982/ip-liberator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
